
import graph.*;
import dbConnect.*;

/**
 * Die Klasse erzeugt ein Graph-Objekt, indem die Knoten und Kanten aus zwei Tabellen einer Datenbank erzeugt werden.
 */ 
public class BABGraph extends Graph{
	private DatabaseConnector babDatenbank;

	
/**
 * Konstruktur der KLasse BABGraph. 
 * Erzeugt den Graphen
 */
	public BABGraph() {
		// TODO Auto-generated constructor stub
        super();
		babDatenbank = new DatabaseConnector("localhost", "3306", "Autobahnen",
				"root", "");

		knotenEinfuegen();
		kantenEinfuegen();
		babDatenbank.close();
	}


	
/**
 * Hilfsmethode, die eine als Parameter übergebenen SQL-Abfrage an die im Konstruktor geoeffnete Datenbank sendet und 
 * das Ergebnis in Form einer String-Tabelle zurückliefert.
 */	
	private String[][] getAbfrageDaten(String abfrage){
		babDatenbank.executeStatement(abfrage);
		QueryResult as = babDatenbank.getCurrentQueryResult();
		if (babDatenbank.getErrorMessage()==null) {
		    return as.getData();
		  } else {
		      return null;
		   }
	}
	
	
	/**
	 * liest die Tabelle 'BABAnschluesse' aus der Datenbank 'Autobahnen' ein und erzeugt 
	 * für alle Anschlussstellen 
	 * einen Knoten des Graphen.
	 */
	public void knotenEinfuegen() {
		String abfrage = "SELECT * FROM BABAnschluesse";
		String[][] babDaten = getAbfrageDaten(abfrage);
		for (int zaehler = 0; zaehler < babDaten.length; zaehler++) {
			String babNr = babDaten[zaehler][1].trim();//Nachfolgende Leerzeichen abschneiden
			String asNr = babDaten[zaehler][2].trim();
			String asTyp = babDaten[zaehler][3].trim();
			String bezeichnung = babDaten[zaehler][4].trim();
            //Bei Kreuzen oder Dreicken wird geprüft, ob schon ein Knoten erzeugt wurde
			if (asTyp.equals("AK") || asTyp.equals("AD")) {
				KreuzVertex kreuzKnoten = (KreuzVertex)this.getVertex(bezeichnung);
				if (kreuzKnoten == null) {
					// neuen Knoten einfuegen
					kreuzKnoten = new KreuzVertex(bezeichnung, babNr);
					this.addVertex(kreuzKnoten);
				} else {
					// zweite Autobahnnummer in vorhandenen Knoten einfuegen
					kreuzKnoten.setzeBAB2(babNr);
				}
			} else {
				// normale Abfahrt einfuegen
				ASVertex abfahrt = new ASVertex(bezeichnung, babNr, asNr);
				this.addVertex(abfahrt);
			}

		}
	}

	
	/**
	 * Liest die Tabelle 'Entfernungen' aus der Datenbank 'Autobahnen' ein und erzeugt für jeden Datensatz
	 * einen Kante zwischen den beiden Endknoten im Graphen.
	 */		
	public void kantenEinfuegen() {
		String abfrage = "SELECT * FROM Entfernungen";
		String[][] babDaten = getAbfrageDaten(abfrage);
		for (int zaehler = 0; zaehler < babDaten.length; zaehler++) {
            String kmString = babDaten[zaehler][2];
            double km = Double.parseDouble(kmString);
            //Bezeichnung zu ID1  in der Datenbank suchen
		    abfrage = "SELECT Bezeichnung FROM BABAnschluesse, Entfernungen WHERE ID = "+babDaten[zaehler][0];
			String[][] abfahrt = getAbfrageDaten(abfrage);
			String as1 = abfahrt[0][0].trim();
            //Bezeichnung zu ID2 in der Datenbank suchen
			abfrage = "SELECT Bezeichnung FROM BABAnschluesse WHERE ID = "+babDaten[zaehler][1];
			abfahrt = getAbfrageDaten(abfrage);		
		    String as2 = abfahrt[0][0].trim();
	        //Knoten im GRaphen suche
			Vertex knoten1 = (Vertex)this.getVertex(as1);
			Vertex knoten2 = (Vertex)this.getVertex(as2);
			if (knoten1!=null && knoten2!=null) {
			    //Kante erzeugen und in Graphen einfügen
			    Edge kante = new Edge(knoten1, knoten2, km);
			    this.addEdge(kante);
			 } else {
			     //Duerfte nie aufgerufen werden
			     System.out.println("Abfahrt "+as1+"oder "+as2+" nicht gefunden");
			 }
		}
	}
}
