 

import java.awt.Container;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import graph.*;
import dijkstra.*;

public class BABRoutenplaner extends JFrame {

	private JButton ausfuehrenKnopf = new JButton();
	private JTextField start;
	private JTextField ziel;
	private JTextArea routenAusgabe;
	private JScrollPane scrollPane;
	private BABGraph autobahnGraph;
	private Dijkstra dij;
	private String ausgabe = "";

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					BABRoutenplaner frame = new BABRoutenplaner();
					frame.setVisible(true);

				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the frame.
	 */
	public BABRoutenplaner() {
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setSize(555, 500);
		Container cp = getContentPane();
		cp.setLayout(null);
		ausfuehrenKnopf.setBounds(35, 443, 140, 33);
		ausfuehrenKnopf.setText("Route suchen");
		ausfuehrenKnopf.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				ausfuehrenKnopf_ActionPerformed(evt);
			}
		});

		cp.add(ausfuehrenKnopf);

		JLabel lblStart = new JLabel("Start");
		lblStart.setBounds(35, 22, 38, 16);
		getContentPane().add(lblStart);

		start = new JTextField();
		start.setBounds(85, 16, 198, 28);
		getContentPane().add(start);
		start.setColumns(10);

		JLabel lblZiel = new JLabel("Ziel");
		lblZiel.setBounds(295, 22, 61, 16);
		getContentPane().add(lblZiel);

		ziel = new JTextField();
		ziel.setBounds(334, 16, 191, 28);
		getContentPane().add(ziel);
		ziel.setColumns(10);

		JLabel lblRoute = new JLabel("Route");
		lblRoute.setBounds(35, 56, 61, 16);
		getContentPane().add(lblRoute);

		routenAusgabe = new JTextArea("");
		scrollPane = new JScrollPane(routenAusgabe);
		scrollPane.setBounds(35, 85, 480, 355);
		getContentPane().add(scrollPane);
		// Ende Komponenten
		setResizable(false);
		this.setVisible(true);
		gaphLaden();
	}

	
	public void gaphLaden(){
		autobahnGraph = new BABGraph();
		dij = new Dijkstra(autobahnGraph);
		routenAusgabe.append("Autobahnen wurden geladen");
   }

	public void ausgabeErzeugen(List<DijkstraVertex> route) {
		BABVertex asV;
		String babNr;
		String abfahrtName;
		route.toFirst();
		routenAusgabe.setText("");
		if (!route.hasAccess()) {
			ausgabe = "Keine Route gefunden";
		} else {
			asV = (BABVertex) route.getContent();
			babNr = asV.gibBABNr();
			abfahrtName = asV.getID();
			ausgabe = "Auffahren A" + babNr + " " + abfahrtName;
			route.next();
			asV = (BABVertex) route.getContent();
			if (asV != null) {
				ausgabe = ausgabe + " Richtung " + asV.getID() + "\n";
				while (route.hasAccess()) {
					abfahrtName = asV.getID();
					if (asV instanceof KreuzVertex) {
					    route.next();
					    asV = (BABVertex) route.getContent();
					    String bab2 = asV.gibBABNr();
					    String richtung;
					    if (asV!=null) {
						   richtung = asV.getID();
						}else {
						    richtung = "Abfahrt";
					    }
						if (!babNr.equals(bab2)) {
							ausgabe = ausgabe + "A" + babNr + " "+ abfahrtName + " auf die A" + bab2+ " Richtung " + richtung + "\n";
							babNr = bab2;
						} else {
							ausgabe = ausgabe + "A" + babNr + " "
									+ abfahrtName + "\n";
							asV = (BABVertex) route.getContent();
						}
					} else {
						route.next();
						asV = (BABVertex) route.getContent();
					}
				}
				route.toLast();
				ASVertex abfahrt = (ASVertex) route.getContent();
				babNr = abfahrt.gibBABNr();
				abfahrtName = abfahrt.getID();
				String asNr = abfahrt.gibASNr();
				ausgabe = ausgabe + "Abfahren A" + babNr + " " + abfahrtName
						+ " AS" + asNr + "\n";
				double weglaenge = dij.gibWeglaenge();
				weglaenge = Math.round(weglaenge * 10) / 10;
				String wegString = String.format("Fahrstrecke: %.1f km",
						weglaenge);
				ausgabe = ausgabe + wegString;
			}
		}
	}

	/**
	 * Erzeugt die Routen-Ausgabe, indem alle passierten Anschlusstellen 
	 * angegeben werden.
	 */
	public void ausgabeErzeugenEinfach(List<DijkstraVertex> route){
		route.toFirst();
		routenAusgabe.setText("");
		while (route.hasAccess()) {
			BABVertex asV = (BABVertex) route.getContent();
			String babNr = asV.gibBABNr();
			String abfahrtName = asV.getID();
		  	ausgabe = ausgabe+babNr + " " + abfahrtName+"\n";
		    route.next();
		}
		double weglaenge = dij.gibWeglaenge();
		weglaenge = Math.round(weglaenge * 10) / 10;
		String wegString = String.format("Fahrstrecke: %.1f km",weglaenge);
		ausgabe = ausgabe + wegString;
	}
	
	
	/**
	 * Berechnet mithilfe des Dijkstra-Algorithmus' den kuerzesten Weg zwischen 
	 * der eingegebenen Auffahrt und der eingegebenen Abfahrt,
	 * sofern passende Knoten im Graphen gefunden werden. 
	 * Anschließend wird ein Text zur Beschreibung der Route formatiert und ausgegeben
	 */
	public void ausfuehrenKnopf_ActionPerformed(ActionEvent evt) {
		dij.zuruecksetzen();
		String startAS = start.getText();
		String zielAS = ziel.getText();
		BABVertex startV = (BABVertex) autobahnGraph.getVertex(startAS);
		BABVertex zielV = (BABVertex) autobahnGraph.getVertex(zielAS);
	    if (startV != null && zielV != null) {
			dij.sucheWeg(startV, zielV);
			List<DijkstraVertex> route = dij.gibRoute();
			ausgabeErzeugen(route);
			routenAusgabe.setText(ausgabe);
	 	} else {
			routenAusgabe.setText("Keine gültigen Anschlussstellen");
		}
	}

}
